#!/usr/bin/env python3

# This script initializes the LaTeX author resources provided for
# Journal of Optimization Theory and Applications (JOTA).
# https://www.springer.com/journal/10957/submission-guidelines

import re
import sys
from ste.utilities import utilities

if __name__ == '__main__':
  try:
    # Remove the initialization time and version stamp.
    utilities.remove_time_version_stamp()

    # Get and unpack the LaTeX author resources from the publisher.
    utilities.get_archive('https://media.springer.com/full/springer-instructions-for-authors-assets/zip/1705114_LaTeX%20JOTA.zip')

    # Fix the missing comment in the spmpsci.bst file.
    data = open('spmpsci.bst', 'r', newline = '\r\n').read()
    data = re.sub(r'`vonx', r'%% `vonx', data)
    open('spmpsci.bst', 'w').write(data)

    # Write the initialization time and version stamp.
    utilities.write_time_version_stamp()

  except:
    sys.exit(1)



import cloudscraper
import datetime
import glob
import importlib.metadata
import importlib.util
import os
import patch
import re
import requests
import shutil
import sys
import tempfile
import setuptools_git_versioning
from subprocess import run, STDOUT


# Define a neutral user agent.
userAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64)"

def get_file(url, verify = True, method = "requests"):
  """
  Retrieves a file from a given url, writes it to the current directory and
  returns the file name.
  """

  if method == "wget":
    # The 'wget' option uses the system-level 'wget' which is capable of
    # overcoming some protections against automated downloads.
    print('Retrieving {0:s} using wget'.format(url))
    filename = url.split("/")[-1]
    filename = filename.split("?")[0]
    print('File name from URL: {0:s}'.format(filename))
    try:
      # We need 'shell = True' likely since some download protection mechanisms
      # check the environment. '--timestamping' is used in order to make wget
      # overwrite possibly existing copies of the file.
      command = 'wget --user-agent="' + userAgent + '" --timestamping --trust-server-names ' + url
      print('Running {0:s}'.format(command))
      run(command, shell = True, stderr = STDOUT)
    except FileNotFoundError:
      print('wget does not seem to be available on your system. Unable to retrieve {0:s}. The template will not be available.'.format(url))

  elif method == "requests":
    # Retrieve the URL using 'requests'.
    print('Retrieving {0:s} using requests'.format(url))
    r = requests.get(url, verify = verify)

    # If the request was rejected, try again using a neutral user agent.
    if not r.ok:
      headers = {"User-Agent": userAgent}
      r = requests.get(url, verify = verify, headers = headers)

    # Retrieve the file name.
    filename = get_filename(r)

    # Write out the raw content.
    with open(filename, 'wb') as f:
      f.write(r.content)

  elif method == "cloudscraper":
    # Retrieve the URL using 'cloudscraper'.
    print('Retrieving {0:s} using cloudscraper'.format(url))
    scraper = cloudscraper.create_scraper()
    r = scraper.get(url, verify = verify)

    # Retrieve the file name.
    filename = get_filename(r)

    # Write out the raw content.
    with open(filename, 'wb') as f:
      f.write(r.content)

  else:
    # Unknown download method.
    print(f'Unknown download method {method}.')
    sys.exit(1)

  # Return the file name.
  return filename


def get_filename(r):
  """
  Retrieves the file name from the request r.
  """
  # Construct the file name either as specified by the server, or else from
  # the URL. The header field 'Content-Disposition', if it exists, typically
  # reads
  #   Content-Disposition: attachment
  #   Content-Disposition: attachment; filename="file.zip"
  #   Content-Disposition: inline; filename="file.zip"

  # Try the 'Content-Disposition' header field.
  filename = r.headers.get("Content-Disposition")
  if filename and re.search('^(attachment|inline)', filename):
    filename = re.findall(r'^(?:attachment|inline); filename=(.*)$', filename)[0]
    # Strip enclosing single or double quotes from the file name.
    filename = filename.strip('\'')
    filename = filename.strip('\"')

  # If this is not successful, extract the file name from the URL.
  if not filename:
    filename = r.url.split("/")[-1]
    filename = filename.split("?")[0]
    print('File name from URL: {0:s}'.format(filename))
  else:
    print('File name from header: {0:s}'.format(filename))

  return filename


def get_archive(url, verify = True, junk = 0, method = "requests"):
  """
  Retrieves an archive file from a given url, extracts it and writes the
  content to the current directory with the required number of leading
  directories junked from the path.
  """

  # Retrieve the archive file.
  filename = get_file(url, verify = verify, method = method)

  # Unpack the archive into a temporary directory (plus whatever relative paths
  # there may be associated with the archive's content).
  tmpdir = 'tmp'
  print('Unpacking {0:s} into {1:s}'.format(filename, tmpdir))
  shutil.unpack_archive(filename, tmpdir)

  # Move the archive's content into the current directory, i.e., junk the
  # tmpdir and whichever number of additional path components as indicated
  # through the junk parameter.
  for f in glob.glob(f'{tmpdir}/**/*', recursive = True):
    
    # Proceed only if the object is a file (not a directory).
    if os.path.isfile(f):
      # Construct the relative path of the destination.
      # Remove the tmpdir/ component from the left.
      regex = r'^' + tmpdir + r'/'
      destination = f
      destination = re.sub(regex, '', destination, count = 1)

      # Remove up to 'junk' many additional path components from the left.
      if junk > 0:
        destination = re.sub(r'.*?/', '', destination, count = junk)

      # Construct the full path of the destination.
      # This may be necessary in order to allow files to be overwritten.
      destination = os.path.join(os.getcwd(), destination)

      # Make sure the destination directory exists.
      if not os.path.exists(os.path.dirname(destination)):
        os.mkdir(os.path.dirname(destination))

      # Move the file.
      print('Moving {0:s} to {1:s}'.format(f, destination))
      shutil.move(f, destination)

  # Remove the archive folder
  shutil.rmtree(tmpdir)


def apply_patch(patchFile):
  """
  Applies the patches described in the patch file.
  """
  # Since https://pypi.org/project/patch/ modifies the source file in place and
  # does not honor the target file, we create a work around. For each patch
  # item, we copy its source file into a temporary file, then apply the patch
  # file, move the patched source files to their destination and move the
  # temporary copies back to the source files.

  # Read the patch data.
  patchData = patch.fromfile(patchFile)

  # Create lists of source, target, and temporary file names.
  sourceFiles = []
  tempFiles = []
  targetFiles = []

  # Iterate over the files to be patched.
  for patchItem in patchData.items:

    # Get and store the files affected by the current patch item.
    sourceFile = patchItem.source
    targetFile = patchItem.target
    tempFile = tempfile.mkstemp()[1]

    # Copy the source file to the temporary file.
    shutil.copy(sourceFile, tempFile)

    # Store the file names.
    sourceFiles.append(sourceFile)
    targetFiles.append(targetFile)
    tempFiles.append(tempFile)

  # Apply the patch set to all files.
  patchData.apply()

  # Iterate over the files again.
  for sourceFile, targetFile, tempFile in zip(sourceFiles, targetFiles, tempFiles):
    # Move the patched source file to its target.
    shutil.move(sourceFile, targetFile)

    # Retrieve the original source file from the temporary file.
    shutil.move(tempFile, sourceFile)


def link_or_copy(source, destination):
  """
  Tries to make destination a symbolic link pointing to source.
  If this fails, copies destination to source instead.
  """
  # Remove the destination if it exists.
  if os.path.isfile(destination):
    os.remove(destination)

  # Try to create a symbolic link.
  # If this fails, copy the source to the destination instead.
  try:
    os.symlink(source, destination)
  except:
    shutil.copy(source, destination)


def protect(source):
  """
  Renames the source file to a reproducible but unlikely name to protect it from being
  overwritten by content of the same name in the publisher's archive.
  """
  destination = 'scoop-tmp-' + source
  os.replace(source, destination)


def restore(source):
  """
  Renames the source file to a reproducible alternative name with prefix
  'scoop-disambiguation-' and restores the original file.
  """
  original = 'scoop-tmp-' + source
  alternative = 'scoop-disambiguation-' + source
  os.replace(source, alternative)
  os.replace(original, source)


def remove_time_version_stamp():
  """
  Remove an existing time and version stamp.
  """
  filename = 'SCOOP-STAMP'
  if os.path.isfile(filename):
    os.remove(filename)


def write_time_version_stamp():
  """
  Write a time and version stamp to indicate successful completion.
  """
  filename = 'SCOOP-STAMP'
  if importlib.util.find_spec("scoop-template-engine") is not None:
    # Get the version number from the package data.
    version = importlib.metadata.version("scoop-template-engine")
  else:
    # Get the version number from the most recent git tag,
    # defaulting to "0.0.0".
    version = str(setuptools_git_versioning.get_tag() or "0.0.0").lstrip("v")

  # Generate the date string.
  dateTime = datetime.datetime.utcnow().strftime("%Y%m%d-%H:%M:%S UTC")

  # Write the time and version stamp.
  stampString = "{version:s}\n{dateTime:s}\n".format(version = version, dateTime = dateTime)
  with open(filename, 'w') as f:
    f.write(stampString)



#!/usr/bin/env python3

# This script initializes the LaTeX author resources provided for
# Springer journals based on the svjour3.cls document class.
# https://www.springer.com/journal/245/submission-guidelines 

import sys
from ste.utilities import utilities

if __name__ == '__main__':
  try:
    # Remove the initialization time and version stamp.
    utilities.remove_time_version_stamp()

    # Get and unpack the LaTeX author resources from the publisher.
    utilities.get_archive('https://media.springer.com/full/springer-instructions-for-authors-assets/zip/468198_LaTeX_DL_468198_240419.zip', junk = 1)

    # Write the initialization time and version stamp.
    utilities.write_time_version_stamp()

  except:
    sys.exit(1)



#!/usr/bin/env python3

# This script initializes the LaTeX author resources provided for
# BIT Numerical Mathematics (BIT).
# https://projects.mai.liu.se/BIT/Contributions.html

import os
import sys
from ste.utilities import utilities
from subprocess import run, STDOUT

if __name__ == '__main__':
  try:
    # Remove the initialization time and version stamp.
    utilities.remove_time_version_stamp()

    # Get the LaTeX author resources from the publisher.
    utilities.get_file('https://projects.mai.liu.se/BIT/BITtemplate.tex')
    utilities.get_file('https://projects.mai.liu.se/BIT/svjour3.cls')
    utilities.get_file('https://projects.mai.liu.se/BIT/svglov3.clo')
    utilities.get_file('https://projects.mai.liu.se/BIT/usrguid3.pdf')

    # Link to spmpsci.bst which is recommended but not available from the author resources page.
    if os.path.isfile('spmpsci.bst'):
      os.remove('spmpsci.bst')
    os.symlink('../svjour3.cls/spmpsci.bst', 'spmpsci.bst')

    # Compile the template file.
    run(['latexmk', '-pdf', '-norc', '-interaction=nonstopmode',  'BITtemplate.tex'], stderr = STDOUT)
    run(['latexmk', '-c'  , '-norc', '-interaction=nonstopmode',  'BITtemplate.tex'], stderr = STDOUT)

    # Write the initialization time and version stamp.
    utilities.write_time_version_stamp()

  except:
    sys.exit(1)



"""
This script prepares a legacy BibTeX file from one or several BibLaTeX 
sources. In particular, it
* converts UTF8 characters into their LaTeX transcriptions
* converts entry types such as @THESIS into BibTeX compatible types,
 depending on command line parameters.
This script can operate 
* in a document oriented mode, where the bibliography is drawn from a 
 document's .bcf file (using the data source(s) declared there), or
* in a database oriented mode, where an entire .bib file is 
 processed.
"""

# Resolve the dependencies.
import argparse
import bibtexparser
import datetime
import importlib.metadata 
import os
import re
import subprocess
import sys
import tempfile


def main():
  """
  implements the user interface to the scoop prepare bibtex file function.
  """

  # Remember who we are and how we were called.
  thisScriptName = os.path.basename(sys.argv[0])
  thisScriptAbsolutePath = os.path.abspath(sys.argv[0])
  thisScriptCallSummary = " ".join([thisScriptName] + sys.argv[1:])

  # Get the version number.
  try:
    scoopTemplateEngineVersion = importlib.metadata.version("scoop-template-engine")
  except:
    scoopTemplateEngineVersion = "VERSION ERROR"

  # Define the command line argument parser.
  parser = argparse.ArgumentParser(description = 'The scoop prepare bibtex engine version {version:s}.'.format(version = scoopTemplateEngineVersion), formatter_class = argparse.RawTextHelpFormatter)
  parser.add_argument('infile', metavar = 'infile', help = 'the .bcf or .bib file to work on.')
  parser.add_argument('outfile', metavar = 'outfile', nargs = '?', help = 'the .bib file to write the output to (default: stdout)')
  parser.add_argument('--giveninits', help = 'abbreviate authors\' and editors\' given names', action = 'store_true')
  parser.add_argument('--onlinetotechreport', help = 'transcribe @ONLINE entries into @TECHREPORT', action = 'store_true')
  parser.add_argument('--proceedingstocollection', help = 'transcribe @PROCEEDINGS entries into @COLLECTION', action = 'store_true')
  parser.add_argument('--doitonote', help = 'transcribe DOI into NOTE fields, with a link to https://doi.org/doi', action = 'store_true')
  parser.add_argument('--doitourl', help = 'transcribe DOI into URL fields, pointing to https://doi.org/doi', action = 'store_true')
  parser.add_argument('--urltonote', help = 'transcribe URL into NOTE fields, with a link to the url', action = 'store_true')
  parser.add_argument('--arxivtotypeornote', help = 'transcribe EPRINTTYPE = {arxiv} into a TYPE (for @TECHREPORTs) or NOTE field', action = 'store_true')
  parser.add_argument('--haltotypeornote', help = 'transcribe EPRINTTYPE = {HAL} into a TYPE (for @TECHREPORTs) or NOTE field', action = 'store_true')
  parser.add_argument('--urntonote', help = 'transcribe EPRINTTYPE = {urn} into a NOTE field', action = 'store_true')
  parser.add_argument('--proceedingstitletobooktitle', help = 'transcribe TITLE into a BOOKTITLE field for @PROCEEDINGS', action = 'store_true')
  parser.add_argument('--version', help = 'write out the version number and exit', action = 'store_true')
  parser.add_argument('--quiet', help = 'do not report anything', action = 'store_true')
  args = parser.parse_args()

  # Define a print function which honors the --quiet option.
  quietprint = print if not args.quiet else lambda *args, **kwargs: None

  # If --version is given, report the version number and exit.
  if args.version:
    print(scoopTemplateEngineVersion)
    sys.exit(0)

  # Make sure the input file exists and is readable.
  try:
    with open(args.infile) as infileStream:
      infileData = infileStream.read()
  except IOError:
    print()
    print('ERROR: Input file {file:s} is not readable.'.format(file = args.infile))
    sys.exit(1)

  # Determine whether we are in document or database oriented mode.
  infileExtension = os.path.splitext(args.infile)[-1]
  if infileExtension == '.bcf':
    mode = 'document'
  elif infileExtension == '.bib':
    mode = 'database'
  else:
    print()
    print('ERROR: Input file {file:s} must have .bcf or .bib extension.'.format(file = args.infile))
    sys.exit(1)

  # Prepare a temporary output .bib file in the /tmp directory.
  temporaryOutfile = tempfile.NamedTemporaryFile(suffix = '.bib')

  # Invoke biber to prepare an initial output .bib file, with UTF8
  # characters replaced by their LaTeX equivalents. Notice that biber also
  # pretty-prints the output, making sure all fields are on one line and white
  # spaces are trimmed.
  # Assemble the biber command line string.
  commandString = 'biber --quiet'
  if mode == 'database':
    commandString += ' --tool'
  commandString += (' --output-safechars --output-format=bibtex --output-file={outfilename:s} {infilename:s}').format(outfilename = temporaryOutfile.name, infilename = args.infile)

  # Invoke biber and make sure its run was successful.
  returnValue = subprocess.run(commandString, shell = True, stdout = subprocess.DEVNULL, stderr = subprocess.DEVNULL)
  try:
    returnValue.check_returncode()
  except subprocess.CalledProcessError:
    print('WARNING: {commandString:s} failed.\n'.format(commandString = commandString))

  # Customize the BibTeX parser.
  parser = bibtexparser.bparser.BibTexParser()
  parser.ignore_nonstandard_types = False

  # Read the .bib file into a dictionary.
  if os.path.getsize(temporaryOutfile.name) == 0:
    print('NOTICE: The input file is empty, either because your document contains no citations, or due to an error.', file = sys.stderr)
    sys.exit(0)
  with open(temporaryOutfile.name) as bibfile:
    bibData = bibtexparser.load(bibfile, parser)

  # Iterate over the bibData entries and modify them as required.
  for entry in bibData.entries:
    # entry.ID represents the cite key as a string.
    # entry.ENTRYTYPE represents the record's type (such as 'ARTICLE').

    # Convert all DATE to YEAR fields, preserving the first four digits of the
    # actual date.
    if entry.get('date') is not None:
      entry['year'] = entry.pop('date')[:4]

    # Replace all LOCATION (BibLaTeX) by ADDRESS (BibTeX) fields.
    if entry.get('location') is not None:
      entry['address'] = entry.pop('location')

    # Replace all JOURNALTITLE (BibLaTeX) by JOURNAL (BibTeX) fields.
    if entry.get('journaltitle') is not None:
      entry['journal'] = entry.pop('journaltitle')

    # Replace all ORGANIZATION (BibLaTeX) by PUBLISHER (BibTeX) fields.
    if entry.get('organization') is not None:
      entry['publisher'] = entry.pop('organization')

    # Append the content of SUBTITLE (BibLaTeX) field to the TITLE (BibTeX) field.
    # Leave SUBTITLE in since it does no harm.
    if entry.get('subtitle') is not None:
      entry['title'] = '. '.join(filter(None, [entry.get('title'), entry.get('subtitle')]))

    # Replace all @REPORT (BibLaTeX) by @TECHREPORT (BibTeX) entries.
    if entry.get('ENTRYTYPE') == 'report':
      entry['ENTRYTYPE'] = 'techreport'

    # Replace all @COLLECTION (BibLaTeX) by @BOOK (BibTeX) entries.
    if entry.get('ENTRYTYPE') == 'collection':
      entry['ENTRYTYPE'] = 'book'

    # Convert @THESIS with TYPE = "Bachelor thesis" (and similar) to @MASTERSTHESIS, and modify the
    # TYPE to say 'Bachelor thesis' explicitly. Replace INSTITUTION by SCHOOL.
    if entry.get('ENTRYTYPE') == 'thesis' and re.search('(Bachelor|B.Sc.|BSc)\s*Thesis', entry.get('type'), flags = re.IGNORECASE) is not None:
      entry['ENTRYTYPE'] = 'mastersthesis'
      entry['type'] = '{B}achelor thesis'
      if entry.get('institution') is not None:
        entry['school'] = entry.pop('institution')

    # Convert @THESIS with TYPE = "mathesis" (and similar) to @MASTERSTHESIS, and modify the
    # TYPE to say 'Master thesis' explicitly. Replace INSTITUTION by SCHOOL.
    if entry.get('ENTRYTYPE') == 'thesis' and re.search('(Master|M.Sc.|MSc|M.A.|MA)\s*Thesis', entry.get('type'), flags = re.IGNORECASE) is not None:
      entry['ENTRYTYPE'] = 'mastersthesis'
      entry['type'] = '{M}aster thesis'
      if entry.get('institution') is not None:
        entry['school'] = entry.pop('institution')

    # Convert @THESIS with TYPE = "phdthesis" (and similar) to @PHDTHESIS, and modify the
    # TYPE to say 'Ph.D. thesis' explicitly. Replace INSTITUTION by SCHOOL.
    if entry.get('ENTRYTYPE') == 'thesis' and re.search('(Doctoral|Ph.D.|PHD)\s*Thesis', entry.get('type'), flags = re.IGNORECASE) is not None:
      entry['ENTRYTYPE'] = 'phdthesis'
      entry['type'] = '{Ph.D.} thesis'
      if entry.get('institution') is not None:
        entry['school'] = entry.pop('institution')

    # Convert @THESIS with TYPE = "Habilitation thesis" (and similar) to @PHDTHESIS, and modify the
    # TYPE to say 'Ph.D. thesis' explicitly. Replace INSTITUTION by SCHOOL.
    if entry.get('ENTRYTYPE') == 'thesis' and re.search('(Habilitation)\s*Thesis', entry.get('type'), flags = re.IGNORECASE) is not None:
      entry['ENTRYTYPE'] = 'phdthesis'
      entry['type'] = '{H}abilitation thesis'
      if entry.get('institution') is not None:
        entry['school'] = entry.pop('institution')


    # Additional conversions triggered by command line switches follow.

    # If required, transcribe @ONLINE into @TECHREPORT entries.
    if args.onlinetotechreport:
      if entry.get('ENTRYTYPE') == 'online':
        entry['ENTRYTYPE'] = 'techreport'

    # If required, transcribe @PROCEEDINGS into @COLLECTION entries.
    if args.proceedingstocollection:
      if entry.get('ENTRYTYPE') == 'proceedings':
        entry['ENTRYTYPE'] = 'collection'

    # If required, transcribe EPRINTTTYPE = {arXiv} fields.
    # In @TECHREPORT, generate a TYPE field from EPRINT field.
    # In all other entry types (for example, @ARTICLE), generate a NOTE field from EPRINT field.
    # Leave EPRINTTYPE in since it does no harm but remove EPRINT since some .bst files 
    # interpret every EPRINT as an arxiv preprint.
    if args.arxivtotypeornote:
      if entry.get('eprinttype') == 'arXiv':
        arxivIdentifier = entry.pop('eprint')
        if arxivIdentifier is not None:
          arxivString = '{{arXiv}}: \href{{https://arxiv.org/abs/{arxivIdentifier:s}}}{{\detokenize{{{arxivIdentifier:s}}}}}'.format(arxivIdentifier = arxivIdentifier)
          if entry.get('ENTRYTYPE') == 'techreport':
            entry['type'] = '. '.join(filter(None, [entry.get('type'), arxivString]))
          else:
            entry['note'] = '. '.join(filter(None, [entry.get('note'), arxivString]))

    # If required, transcribe EPRINTTTYPE = {HAL} fields.
    # In @TECHREPORT, generate a TYPE field from the EPRINT field.
    # In all other entry types (for example, @ARTICLE), generate a NOTE field from EPRINT field.
    # Leave EPRINTTYPE in since it does no harm but remove EPRINT since some .bst files 
    # interpret every EPRINT as an arxiv preprint.
    if args.haltotypeornote:
      if entry.get('eprinttype') == 'HAL':
        halIdentifier = entry.pop('eprint')
        if halIdentifier is not None:
          halString = '{{HAL}}: \href{{https://hal.archives-ouvertes.fr/{halIdentifier:s}}}{{\detokenize{{{halIdentifier:s}}}}}'.format(halIdentifier = halIdentifier)
          if entry.get('ENTRYTYPE') == 'techreport':
            entry['type'] = '. '.join(filter(None, [entry.get('type'), halString]))
          else:
            entry['note'] = '. '.join(filter(None, [entry.get('note'), halString]))

    # If required, transcribe EPRINTTTYPE = {urn} fields.
    # In @TECHREPORT, generate a TYPE field from the EPRINT field.
    # In all entry types (for example, @ARTICLE), generate a NOTE field from EPRINT field.
    # Leave EPRINTTYPE in since it does no harm but remove EPRINT since some .bst files 
    # interpret every EPRINT as an arxiv preprint.
    if args.urntonote:
      if entry.get('eprinttype') == 'urn':
        urnIdentifier = entry.pop('eprint')
        if urnIdentifier is not None:
          urnString = '{{URN}}: \href{{https://www.nbn-resolving.de/{urnIdentifier:s}}}{{\detokenize{{{urnIdentifier:s}}}}}'.format(urnIdentifier = urnIdentifier)
          entry['note'] = '. '.join(filter(None, [entry.get('note'), urnString]))

    # If required, transcribe DOI into URL fields.
    # Leave DOI in since it does no harm.
    if args.doitourl:
      doi = entry.get('doi') 
      if doi is not None:
        urlString = 'https://doi.org/{doi:s}'.format(doi = doi)
        entry['url'] = '. '.join(filter(None, [entry.get('url'), urlString]))

    # If required, transcribe DOI into NOTE fields.
    # Leave DOI in since it does no harm.
    if args.doitonote:
      doi = entry.get('doi') 
      if doi is not None:
        noteString = '{{DOI}} \href{{https://doi.org/{doi:s}}}{{\detokenize{{{doi:s}}}}}'.format(doi = doi)
        entry['note'] = '. '.join(filter(None, [entry.get('note'), noteString]))

    # If required, transcribe URL into NOTE fields.
    # Leave DOI in since it does no harm.
    if args.urltonote:
      url = entry.get('url') 
      if url is not None:
        noteString = '\\url{{{url:s}}}'.format(url = url)
        entry['note'] = '. '.join(filter(None, [entry.get('note'), noteString]))

    # If required, transcribe TITLE into BOOKTITLE fields for @PROCEEDINGS.
    if args.proceedingstitletobooktitle:
      if entry.get('ENTRYTYPE') == 'proceedings':
        title = entry.pop('title')
        if title is not None:
          entry['booktitle'] = title

    # If required, abbreviate authors' and editors' given names.
    if args.giveninits:
      def abbreviateFullName(fullName):
        # Split names (which always come in comma-separated form) such as 
        #   'Smith, John'
        #   'Smith, Jr, John'
        # into their components, using the comma as a separator. Then process
        # the last component (the given names) through the abbreviate function, and
        # paste the results back together.
        nameParts = fullName.split(',')
        return ', '.join(nameParts[:-1] + [abbreviateGivenNames(nameParts[-1].strip())])
      def abbreviateGivenNames(givenNames):
        # Replace (repeated) whitespaces in givenNames by a single ' '.
        # Then split the givenNames at ' ' and '-', apply abbreviateSingleGivenName to
        # each part, and paste the results back together, using the captured separators.
        givenNames = ' '.join(givenNames.split())
        givenNames = re.split('([ -])', givenNames)
        for iter in range(0,len(givenNames),2):
          givenNames[iter] = abbreviateSingleGivenName(givenNames[iter])
        return ''.join(givenNames)

      def abbreviateSingleGivenName(givenName):
        # Abbreviate a single given name, taking into account (partly fictitious) cases such as
        #   'Donald'
        #   'Jean-Paul'
        #   '\.{I}lker'
        #   '\v{R}\'{\i{}}'
        #   '\AE{}'
        # We use the following logic. If the single given name starts with '\', then copy it
        # until we have found at least one '{' and then until the matching '}'. If the single
        # name does not start with '\', the copy only the first character.
        if givenName[0] == '\\':
          numberOfOpeningBrackets = 0;
          numberOfClosingBrackets = 0;
          for iter in range(len(givenName)):
            if givenName[iter] == '{': 
              numberOfOpeningBrackets += 1
            if givenName[iter] == '}': 
              numberOfClosingBrackets += 1
            if (numberOfOpeningBrackets > 0) and (numberOfOpeningBrackets == numberOfClosingBrackets): 
              break
          return givenName[:iter+1] + '.'
        else:
          return givenName[0] + '.'

      # Perform the abbreviations of author and editor names.
      if entry.get('author') is not None:
        entry['author'] = ' and '.join([abbreviateFullName(authorName) for authorName in entry.get('author').split(' and ')])
      if entry.get('editor') is not None:
        entry['editor'] = ' and '.join([abbreviateFullName(editorName) for editorName in entry.get('editor').split(' and ')])

    # Protect upper-case characters.
    # In all TITLE, SUBTITLE, BOOKTITLE fields, protect all consecutive chains of
    # uppercase letters by braces.
    def protect(string):
      return re.sub(r"([A-Z]+)", r"{\1}", string)
    if entry.get('title') is not None:
      entry['title'] = protect(entry.get('title'))
    if entry.get('subtitle') is not None:
      entry['subtitle'] = protect(entry.get('subtitle'))
    if entry.get('booktitle') is not None:
      entry['booktitle'] = protect(entry.get('booktitle'))

  # Prepare a custom writer.
  writer = bibtexparser.bwriter.BibTexWriter()
  writer.indent = '  '
  writer.add_trailing_comma = True

  # Write the pybtex dictionary into a string initially.
  bibDataString = writer.write(bibData)

  # Create a time and invokation stamp.
  stampString = """@COMMENT{{
Generated by {scriptName:s} (version {version:s})
on {dateTime:s} using
{callSummary:s}
}}

""".format(
    scriptName = thisScriptName, 
    version = scoopTemplateEngineVersion, 
    dateTime = datetime.datetime.utcnow().strftime("%Y%m%d-%H:%M:%S UTC"), 
    callSummary = thisScriptCallSummary)

  # Finally, write the result to the desired outfile.
  if args.outfile is not None:
    with open(args.outfile, 'w') as outfile:
      outfile.write(stampString + bibDataString)
  else:
    print(stampString + bibDataString)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3

# This script initializes the LaTeX author resources provided for
# Springer journals based on the bmcart.cls document class.
# https://advancesincontinuousanddiscretemodels.springeropen.com/submission-guidelines/preparing-your-manuscript
# https://bmcbioinformatics.biomedcentral.com/submission-guidelines/preparing-your-manuscript

import sys
from ste.utilities import utilities

if __name__ == '__main__':
  try:
    # Remove the initialization time and version stamp.
    utilities.remove_time_version_stamp()

    # Get and unpack the LaTeX author resources from the publisher.
    utilities.get_archive('https://resource-cms.springernature.com/springer-cms/rest/v1/content/18361120/data/v2', junk = 1)

    # Write the initialization time and version stamp.
    utilities.write_time_version_stamp()

  except:
    sys.exit(1)
